% --- Source file: SAMC.adapt.Rd ---
\name{SAMC.adapt}
\alias{SAMC.adapt}
\title{SAMC (Stochastic Approximation Monte Carlo) method for small p-value estimation}
\description{
  To estimate the p-value for an observed test statistic based on the 
     SAMC (Stochastic Approximation Monte Carlo) method. 
}
\usage{
SAMC.adapt(data.input, t.obs, t.start=0, n.iter.1=200000, n.iter.2=1000000, 
      n.region.1=101, n.region.2=301, prop.change=0.05, gain.factor.t0=1000, 
      fun.test.statistic=NULL, fun.proposal=NULL, ...)
}
\arguments{
  \item{data.input}{The input data, which can be any data object, e.g. data frame, list, etc., as long as it
     is compatible with the two user provided functions \code{fun.test.statistic} and \code{fun.proposal}. }
  \item{t.obs}{The threshold at which the tail probability is to be calculated. For example, this value 
       can be set to the observed test statistic. The estimated p-value for this value will be computed.}
  \item{t.start}{The minimum value for the considered test statistic. The default is 0.}
  \item{n.iter.1}{The number of iterations used for the initial run with subregions defined uniformly
      in the interval [\code{t.start}, \code{t.obs}], with final region [\code{t.obs}, \eqn{\infty}{\Inf}).
        The default is 200000.}
  \item{n.iter.2}{The number of iterations used for the final run with subregions defined 
       according to the results from the initial run. If \code{n.iter.2} = 0, then only the initial
      run is performed. The default is 1000000. }
  \item{n.region.1}{The number of subregions used by SAMC in the initial run. The default is 101.}
  \item{n.region.2}{The number of subregions used by SAMC in the final run. The default is 301.}
  \item{prop.change}{The proportion of data to be updated by \code{fun.proposal} in each iteration. The default is 0.05.}
  \item{gain.factor.t0}{This is used for defining the gain factor sequence. The default is 1000.}
  \item{fun.test.statistic}{A function written by the user to calculate and return the test statistic based on
         \code{data.input}. It has the following format: \code{fun.test.statistic(data.input, ...)}. 
         See the example function \code{t.test.statistic}.}
  \item{fun.proposal}{A function written by the user to update the input data and return the updated data.
        It should be in the following format: \code{fun.proposal(data.input, prop.change, ...)}.
        See the example function \code{proposal.permute.vector}.}
  \item{...}{Other arguments for \code{fun.test.statistic} and/or \code{fun.proposal}}
}
\value{
  A list with two components \code{hist.mat} and \code{p.value}. The object \code{hist.mat}
  will be a data frame of dimension \code{n.region.2} by 4 (or \code{n.region.1} by 4 if \code{n.iter.2} = 0),
   which contains information used for checking convergence. The other component of each sublist is \code{p.value},
   which is the estimated p-value.
  The \code{samplefreq} column of \code{hist.mat} provides the number of times the SAMC iterations generate a sample falling into each individual sub-region.
   A barplot of \code{samplefreq} (barplot(hist.mat[, "samplefreq"]) should show a flat pattern.
 }
%\details{  
%} % END: details
\references{ 
Yu K, Liang F, Ciampa J, Chatterjee N. Efficient p-value evaluation for resampling-based tests (Submitted manuscript).
 }
\author{Kai Yu }
\seealso{ \code{\link{t.test.statistic}}, \code{\link{proposal.permute.vector}}, \code{\link{SAMC.fun}} }
\examples{
# Example 1: An example of two-sample t test. 
# Suppose we do not know the theoretical distribution of t test statistic, 
# but instead want to rely on a permutation procedure to evaluate the significance 
# level for a given observed statistic value. In this permutation procedure, we 
# randomly shuffle the group IDs among the two samples and generate new versions of 
# the data. We can apply the SAMC algorithm to estimate the p-value (tail probability)
# at a much reduced number of iterations than the standard permutation procedure.
# First, we need to define the two functions for running SAMC, fun.test.statistic,
# and fun.proposal. In the SPERT package, we provide such two functions. They are
# t.test.statistic, which calculates the two sided t-test statistic, and 
# proposal.permute.vector, which update the data through permutation. 

 set.seed(1)
x<-rnorm(200, mean=0, sd=1)
y<-rnorm(200, mean=0.5, sd=1)
data.input<-list(x=x, y=y)
t.obs<-t.test.statistic(data.input)

# True p-value
p.true <- 2.0*(1-pt(t.obs, df=398))
p.true

# Run SAMC to estimate the p-value
res<-SAMC.adapt(data.input, t.obs, t.start=0, n.iter.1=5000, 
            n.iter.2=10000, n.region.1=101, n.region.2=201,
         prop.change=0.05, gain.factor.t0=1000, 
         fun.test.statistic=t.test.statistic, fun.proposal=proposal.permute.vector)

# The estimated p-value
res$p.value

# To check for convergence:
barplot(res$hist.mat[, "samplefreq"])

# Note that the barplot is not flat, so the algorithm has not converged yet. 
# To get a more precise estimate run the code below.
# WARNING: the following call to SAMC.adapt could take around 20 minutes to run.
ptm <- proc.time() 
#res<-SAMC.adapt(data.input, t.obs, t.start=0, n.iter.1=200000, 
#            n.iter.2=1000000, n.region.1=101, n.region.2=301,
#         prop.change=0.05, gain.factor.t0=1000, 
#         fun.test.statistic=t.test.statistic, fun.proposal=proposal.permute.vector)
proc.time()-ptm
}

\keyword{ htest } 

